@@ -3,7 +3,7 @@ from .dataset import Dataset
 from .integration import Integration
 from .config import Config
 from .colors import Colors
-from .network import Network
+from .network import Network, compare_community_jaccard_similarity
 from .cnmf import cNMF
 from .plots import *
 from . import utils, __version__, cpus_available
@@ -13,11 +13,16 @@ import logging
 import subprocess
 import collections
 import sys
+from itertools import product
 from typing import Optional, Mapping
 
+from tqdm import tqdm
 import click
 import pandas as pd
 
+
+CONTEXT_SETTINGS = {'help_option_names': ['-h', '--help']}
+
 class _OrderedGroup(click.Group):
     """
     Overwrites Groups in click to allow ordered commands.
@@ -31,7 +36,7 @@ class _OrderedGroup(click.Group):
         return self.commands
 
 
-@click.group(cls=_OrderedGroup)
+@click.group(cls=_OrderedGroup, context_settings=CONTEXT_SETTINGS)
 @click.version_option(version=__version__)
 def cli():
     """
@@ -47,6 +52,9 @@ def cli():
 @click.option(
     "-m", "--metadata", type=click.Path(dir_okay=False, exists=True), required=False,
     help="Optional delimited text file with metadata for samples/cells with one row each. Columns are annotation layers.")
+@click.option(
+    "-f", "--feature_metadata", type=click.Path(dir_okay=False, exists=True), required=False,
+    help="Optional delimited text file with metadata for features with one row each. Columns are annotation layers.")
 @click.option(
     "--sparsify", is_flag=True,
     help="Save resulting data in sparse format. Recommended to increase performance for sparse datasets such as scRNA-Seq, scATAC-Seq, and 10X Visium, but not for bulk expression data.")
@@ -58,11 +66,11 @@ def cli():
     help="Delimiter for data file, defaults to tab-delimited.")
 @click.option(
     "--metadata_delimiter", type=str, default="\t",
-    help="Delimiter for metadata file, defaults to tab-delimited.")
+    help="Delimiter for metadata files, defaults to tab-delimited.")
 @click.option(
     "-o", '--output', type=click.Path(dir_okay=False, exists=False), required=True,
     help="Path to output .h5ad file.")
-def cmd_txt_to_h5ad(data_file, is_normalized, metadata, output, transpose, sparsify, data_delimiter, metadata_delimiter):
+def cmd_txt_to_h5ad(data_file, is_normalized, metadata, feature_metadata, output, transpose, sparsify, data_delimiter, metadata_delimiter):
     """
     Create .h5ad file with data and metadata (`adata.obs`).
     """
@@ -71,13 +79,17 @@ def cmd_txt_to_h5ad(data_file, is_normalized, metadata, output, transpose, spars
     if transpose:
         df = df.T
     if metadata:
-        metadata_df = pd.read_table(metadata, index_col=0, sep=metadata_delimiter).dropna(axis=1, how="all")
+        sample_metadata_df = pd.read_table(metadata, index_col=0, sep=metadata_delimiter).dropna(axis=1, how="all")
     else:
-        metadata_df = None
-    dataset = Dataset.from_df(data=df, obs=metadata_df, sparsify=sparsify, is_normalized=is_normalized)
-    logging.info("Data types for non-missing values in each layer of metadata:\n"
-                 + dataset.get_metadata_type_summary())
+        sample_metadata_df = None
+    if feature_metadata:
+        feature_metadata_df = pd.read_table(feature_metadata, index_col=0, sep=metadata_delimiter).dropna(axis=1, how="all")
+    else:
+        feature_metadata_df = None
+    dataset = Dataset.from_df(data=df, obs=sample_metadata_df, var=feature_metadata_df, sparsify=sparsify, is_normalized=is_normalized)
+    logging.info(dataset.get_printable_metadata_type_summary())
     dataset.write_h5ad(output)
+    logging.info("All tasks completed successfully.")
 
 @click.command(name="update-h5ad-metadata")
 @click.option(
@@ -94,9 +106,9 @@ def cmd_update_h5ad_metadata(input_h5ad, metadata):
     dataset = Dataset.from_h5ad(input_h5ad)
     metadata_df = pd.read_table(metadata, index_col=0).dropna(axis=1, how="all")
     dataset.update_obs(metadata_df)
-    logging.info("Data types for non-missing values in each layer of metadata:\n"
-                 + dataset.get_metadata_type_summary())
+    logging.info(dataset.get_printable_metadata_type_summary())
     dataset.write_h5ad(input_h5ad)
+    logging.info("All tasks completed successfully.")
 
 @click.command(name="impute-zeros")
 @click.option(
@@ -105,16 +117,21 @@ def cmd_update_h5ad_metadata(input_h5ad, metadata):
 @click.option(
     "-o", "--output", type=click.Path(dir_okay=False, exists=False), required=True,
     help="Output .h5ad file.")
-def cmd_impute_zeros(input, output):
+@click.option(
+    "-n", "--n_folds", type=int, default=100,
+    help="Number of folds for k-fold cross-validation. 0 disables cross-validation.")
+
+def cmd_impute_zeros(input, output, n_folds):
     """
     Impute missing values with zeros.
     """
     utils.start_logging()
-    raise NotImplementedError  # TODO: impute_zeros
     dataset = Dataset.from_h5ad(input)
-    
+    dataset.impute_zeros(n_folds=n_folds)
     # Save output to new h5ad file
     dataset.write_h5ad(output)
+    
+    logging.info("All tasks completed successfully.")
 
 @click.command(name="impute-knn")
 @click.option(
@@ -123,16 +140,37 @@ def cmd_impute_zeros(input, output):
 @click.option(
     "-o", "--output", type=click.Path(dir_okay=False, exists=False), required=True,
     help="Output .h5ad file.")
-def cmd_impute_knn(input, output):
+@click.option(
+    "--n_neighbors", default=5, type=int, show_default=True,
+    help="Number of neighboring samples to use for imputation.")
+@click.option(
+    "--weights", default = "distance", type=str, show_default=True,
+    help="""Weight function used in prediction, defaults to 'distance'. Possible values:
+            
+            - 'uniform' : uniform weights. All points in each neighborhood are
+            weighted equally.
+
+            - 'distance' : weight points by the inverse of their distance.
+            in this case, closer neighbors of a query point will have a
+            greater influence than neighbors which are further away.
+            
+            .
+            """
+)
+@click.option(
+    "-n", "--n_folds", type=int, default=100,
+    help="Number of folds for k-fold cross-validation. 0 disables cross-validation.")
+def cmd_impute_knn(input, output, n_neighbors, weights, n_folds):
     """
-    K-nearest neighbour imputation of missing values.
+    k-Nearest Neighbour (KNN) imputation of missing values.
     """
     utils.start_logging()
-    raise NotImplementedError  #TODO: impute_knn
     dataset = Dataset.from_h5ad(input)
-    
+    dataset.impute_knn(n_neighbors = n_neighbors, weights = weights, n_folds=n_folds)
     # Save output to new h5ad file
     dataset.write_h5ad(output)
+    
+    logging.info("All tasks completed successfully.")
 
 
 @click.command(name="check-h5ad")
@@ -144,16 +182,22 @@ def cmd_impute_knn(input, output):
     help="Output .h5ad file. If not specified, no output file will be written.")
 def cmd_check_h5ad(input, output):
     """
-    Removes unfactorizable genes from a .h5ad file.
+    Removes unfactorizable features (features with zero variance and features with missing values).
+    This step prevents errors during factorization. If missing values are present in a larger
+    number of features, it is recommended to run `mosaicmpi impute_knn` first to impute missing
+    values. Then, this command will remove only those features that have zero variance or were
+    not able to be imputed for other reasons.
     """
     utils.start_logging()
     dataset = Dataset.from_h5ad(input)
-    dataset.remove_unfactorizable_genes()
+    dataset.remove_unfactorizable_features()
     
     # Save output to new h5ad file
     if output is not None:
         dataset.write_h5ad(output)
 
+    logging.info("All tasks completed successfully.")
+
 
 @click.command(name="model-odg")
 @click.option(
@@ -171,7 +215,14 @@ def cmd_check_h5ad(input, output):
 @click.option(
     "--default_dof", type=int, default=20, show_default=True,
     help="Degrees of Freedom (number of components) for the Generalized Additive Model (default method).")
-def cmd_model_odg(name, output_dir, input, default_spline_degree, default_dof):
+@click.option(
+    "--max_missingness", type=float, default=0.0, show_default=True,
+    help="""Maximum proportion of missing values allowed for each feature prior to modelling the mean-variance relationship.
+            This parameter is helpful for reducing the tendency of kNN- and zero-imputed features to have higher variance 
+            relative to unimputed genes.
+            """
+)
+def cmd_model_odg(name, output_dir, input, default_spline_degree, default_dof, max_missingness):
     """
     Model gene overdispersion and plot calibration plots for selection of overdispersed genes, using two methods:
     
@@ -191,8 +242,9 @@ def cmd_model_odg(name, output_dir, input, default_spline_degree, default_dof):
     dataset = Dataset.from_h5ad(input)
     
     # Create gene stats table and save h5ad file
-    dataset.compute_gene_stats(odg_default_spline_degree=default_spline_degree,
-                               odg_default_dof=default_dof)
+    dataset.model_overdispersed_genes(odg_default_spline_degree=default_spline_degree,
+                               odg_default_dof=default_dof,
+                               max_missingness=max_missingness)
     dataset.write_h5ad(os.path.join(output_dir, name, name + ".h5ad"))
     
     # output text file
@@ -202,8 +254,20 @@ def cmd_model_odg(name, output_dir, input, default_spline_degree, default_dof):
 
     # create mean vs variance plots
     fig = plot_feature_dispersion(dataset, show_selected=False)
-    fig.savefig(os.path.join(output_dir, name, "odgenes.pdf"), facecolor='white')
-    fig.savefig(os.path.join(output_dir, name, "odgenes.png"), dpi=400, facecolor='white')
+    fig.savefig(os.path.join(output_dir, name, "feature_meanvar.pdf"), facecolor='white')
+    fig.savefig(os.path.join(output_dir, name, "feature_meanvar.png"), dpi=400, facecolor='white')
+    
+
+    if dataset.is_imputed:
+        logging.info("Creating plots for imputed data")
+
+        fig = plot_feature_missingness(dataset, proportion=True)
+        fig.savefig(os.path.join(output_dir, name, "missingness_histogram.pdf"), facecolor='white')
+        fig.savefig(os.path.join(output_dir, name, "missingness_histogram.png"), dpi=400, facecolor='white')
+
+        fig = plot_feature_dispersion(dataset)
+
+    logging.info("All tasks completed successfully.")
 
 
 @click.command(name="set-parameters")
@@ -321,6 +385,8 @@ def cmd_set_parameters(name, output_dir, odg_method, odg_param, min_mean, k_rang
     # output dataset with new information on overdispersed genes and cNMF parameters
     dataset.write_h5ad(os.path.join(output_dir, name, name + ".h5ad"))
     
+    logging.info("All tasks completed successfully.")
+    
 
 @click.command(name="factorize")
 @click.option(
@@ -338,7 +404,6 @@ def cmd_set_parameters(name, output_dir, odg_method, odg_param, min_mean, k_rang
 @click.option(
     '--slurm_script', type=click.Path(dir_okay=False, exists=True),
     help="Submit jobs to SLURM scheduler using this job submission script. Sample scripts are located in `scripts/slurm.sh`.")
-
 def cmd_factorize(name, output_dir, worker_index, total_workers, slurm_script):
     """
     Performs factorization according to parameters specified using `mosaicmpi set-parameters`.
@@ -349,7 +414,7 @@ def cmd_factorize(name, output_dir, worker_index, total_workers, slurm_script):
     run_params = utils.load_df_from_npz(cnmf_obj.paths['nmf_replicate_parameters'])
     if run_params.shape[0] == 0:
         logging.error("No factorization to do: either no values of k were selected using `mosaicmpi set-parameters` or iterations were set to 0.")
-
+        sys.exit(1)
     if slurm_script is None:
         cnmf_obj.factorize(worker_i=worker_index, total_workers=total_workers)
     else:
@@ -375,34 +440,58 @@ def cmd_factorize(name, output_dir, worker_index, total_workers, slurm_script):
 @click.option(
     '--force_h5ad_update', is_flag=True,
     help="If specified, overwrites cNMF results already saved to the .h5ad file.")
-def cmd_postprocess(name, output_dir, cpus, local_density_threshold, local_neighborhood_size, skip_missing_iterations, force_h5ad_update):
+@click.option(
+    '--slurm_script', type=click.Path(dir_okay=False, exists=True),
+    help="Submit jobs to SLURM scheduler using this job submission script. Sample script is located in `scripts/slurm_postprocess.sh`.")
+def cmd_postprocess(name, output_dir, cpus, local_density_threshold, local_neighborhood_size, skip_missing_iterations, force_h5ad_update, slurm_script):
     """
     Perform post-processing routines on cNMF after factorization. This includes checking factorization outputs for completeness, combining individual
-    iterations, calculating consensus programs and usage matrices, and creating the k-selection and annotated usage plots.
+    iterations, calculating consensus programs and usage matrices, and creating the k-selection plot.
     """
-    cnmf_obj = cNMF(output_dir=output_dir, name=name)
-    utils.start_logging(os.path.join(output_dir, name, "logfile.txt"))
-    cnmf_obj.postprocess(cpus=cpus,
-                         local_density_threshold=local_density_threshold,
-                         local_neighborhood_size=local_neighborhood_size,
-                         skip_missing_iterations=skip_missing_iterations)
-    h5ad_path = os.path.join(output_dir, name, name + ".h5ad")
-    dataset = Dataset.from_h5ad(h5ad_path)
-    
-    cnmf_data_loaded =  "cnmf_usage" in dataset.adata.obsm or\
-                        "cnmf_gep_score" in dataset.adata.varm or\
-                        "cnmf_gep_tpm" in dataset.adata.varm or\
-                        "cnmf_gep_raw" in dataset.adata.varm
-    if cnmf_data_loaded and not force_h5ad_update:
-        logging.Error(f"Error: AnnData already contains cNMF results. Use --force_h5ad_update to overwrite.")
-        sys.exit(1)
 
-    dataset.add_cnmf_results(cnmf_output_dir=output_dir,
-                             cnmf_name=name,
-                             local_density_threshold=local_density_threshold,
-                             local_neighborhood_size=local_neighborhood_size
-                             )
-    dataset.write_h5ad(h5ad_path)
+    if slurm_script is not None:
+        flags = ""
+        if skip_missing_iterations:
+            flags += "--skip_missing_iterations "
+        if force_h5ad_update:
+            flags += "--force_h5ad_update "
+
+        subprocess.Popen(['sbatch', slurm_script,
+                          os.getcwd(),
+                          output_dir,
+                          name,
+                          cpus,
+                          local_density_threshold,
+                          local_neighborhood_size,
+                          skip_missing_iterations,
+                          force_h5ad_update])
+    else:
+        cnmf_obj = cNMF(output_dir=output_dir, name=name)
+        utils.start_logging(os.path.join(output_dir, name, "logfile.txt"))
+        cnmf_obj.postprocess(cpus=cpus,
+                            local_density_threshold=local_density_threshold,
+                            local_neighborhood_size=local_neighborhood_size,
+                            skip_missing_iterations=skip_missing_iterations)
+        h5ad_path = os.path.join(output_dir, name, name + ".h5ad")
+        dataset = Dataset.from_h5ad(h5ad_path)
+        
+        cnmf_data_loaded =  "cnmf_usage" in dataset.adata.obsm or\
+                            "cnmf_gep_score" in dataset.adata.varm or\
+                            "cnmf_gep_tpm" in dataset.adata.varm or\
+                            "cnmf_gep_raw" in dataset.adata.varm
+        if cnmf_data_loaded and not force_h5ad_update:
+            logging.error(f"Error: AnnData already contains cNMF results. Use --force_h5ad_update to overwrite.")
+            sys.exit(1)
+
+        dataset.add_cnmf_results(cnmf_output_dir=output_dir,
+                                cnmf_name=name,
+                                local_density_threshold=local_density_threshold,
+                                local_neighborhood_size=local_neighborhood_size
+                                )
+        dataset.write_h5ad(h5ad_path)
+        
+        logging.info("All tasks completed successfully.")
+
     
 @click.command("annotated-heatmap")
 @click.option(
@@ -412,7 +501,7 @@ def cmd_postprocess(name, output_dir, cpus, local_density_threshold, local_neigh
     help="Output directory for annotated heatmaps.")
 @click.option(
     '-m', '--metadata_colors_toml', type=click.Path(dir_okay=False, exists=True),
-    help="TOML file with metadata_colors specification. See README for more information. If not provided, visually distinct colors will be chosen automatically.")
+    help="TOML file with metadata_colors specification. If not provided, visually distinct colors will be chosen automatically.")
 @click.option(
     '--max_categories_per_layer', type=int,
     help="Filter metadata layers by the number of categories. This parameter is useful to simplify heatmaps with too many annotations.")
@@ -461,6 +550,8 @@ def cmd_annotated_heatmap(input_h5ad, output_dir, metadata_colors_toml, max_cate
             dataset=dataset, k=k, subset_metadata=subset_columns, colors=colors, title=title,
             cluster_samples=True, cluster_programs=False, show_sample_labels=(not hide_sample_labels))
         fig.savefig(filename, transparent=False, bbox_inches = "tight")
+    
+    logging.info("All tasks completed successfully.")
 
 
 @click.command(name="create-config")
@@ -524,7 +615,7 @@ def cmd_integrate(output_dir, config_toml, communities_toml, colors_toml, cpus):
         if "patient_id_col" in dsparams:
             dataset.patient_id_col = dsparams["patient_id_col"]
         datasets[dsname] = dataset
-        
+
     # gets k-values from config
     has_k_subset = {dsname: ("k_subset" in dsparams) for dsname, dsparams in config.datasets.items()}
     if all(list(has_k_subset.values())):
@@ -625,10 +716,10 @@ def cmd_integrate(output_dir, config_toml, communities_toml, colors_toml, cpus):
     
     # Write representative programs and usages
     logging.info("Writing representative programs")
-    rep_programs_ids = network.get_representative_programs()
-    rep_programss = network.integration.get_programs()[rep_programs_ids.index]
-    rep_programss.columns = pd.MultiIndex.from_tuples([[community] + list(program_id) for community, program_id in zip(rep_programs_ids, rep_programs_ids.index)], names=["Community", "dataset", "k", "Program"])
-    rep_programss.to_csv(os.path.join(output_dir, "representative_programs.txt"), sep="\t") # outputs the programs to text file
+    rep_programs_ids = network.get_representative_program_ids()
+    rep_programs = network.integration.get_programs()[rep_programs_ids.index]
+    rep_programs.columns = pd.MultiIndex.from_tuples([[community] + list(program_id) for community, program_id in zip(rep_programs_ids, rep_programs_ids.index)], names=["Community", "dataset", "k", "Program"])
+    rep_programs.to_csv(os.path.join(output_dir, "representative_programs.txt"), sep="\t") # outputs the programs to text file
     rep_programs_usage = network.integration.get_usages()[rep_programs_ids.index]
     rep_programs_usage.columns = pd.MultiIndex.from_tuples([[community] + list(program_id) for community, program_id in zip(rep_programs_ids, rep_programs_ids.index)], names=["Community", "dataset", "k", "Program"])
     rep_programs_usage.to_csv(os.path.join(output_dir, "representative_program_usages.txt"), sep="\t") # outputs the program usages to text file
@@ -687,13 +778,17 @@ def cmd_integrate(output_dir, config_toml, communities_toml, colors_toml, cpus):
         fig = plot_program_network_npatients(network, colors)
         utils.save_fig(fig, os.path.join(output_dir, f"network_n_patients"), target_dpi=600, formats=config.plot_formats)
     
-    logging.info("Creating community usage heatmap...")
 
     # integrated community usage
     ic_usage = network.get_community_usage()
     ic_usage.to_csv(os.path.join(output_dir, "community_usage.txt"), sep="\t")
-    fig = plot_community_usage_heatmap(network, colors)
-    utils.save_fig(fig, os.path.join(output_dir, f"community_usage"), target_dpi=200, formats=config.plot_formats)
+    n_samples = ic_usage.shape[0]
+    if n_samples <= config.max_cells_per_heatmap_dimension:
+        logging.info("Creating community usage heatmap...")
+        fig = plot_community_usage_heatmap(network, colors)
+        utils.save_fig(fig, os.path.join(output_dir, f"community_usage"), target_dpi=200, formats=config.plot_formats)
+    else:
+        logging.info(f"Skipped community usage heatmap. n_samples = {n_samples} is greater than the 'max_cells_per_heatmap_dimension' parameter.")
 
     logging.info("Computing community-level associations")
 
@@ -798,25 +893,31 @@ def cmd_integrate(output_dir, config_toml, communities_toml, colors_toml, cpus):
             layer_str = layer.replace("\\", "_").replace("/", "_")
             fig = plot_metadata_correlation_program_network(network, colors, layer=layer, subset_datasets=dataset_name)   
             utils.save_fig(fig, os.path.join(output_dir, "annotated_programs", "correlation_network", dataset_name, layer_str), target_dpi=600, formats=config.plot_formats)
+
+    logging.info("All tasks completed successfully.")
     
 @click.command(name="ssgsea")
 @click.option('-o', '--output_dir', type=click.Path(file_okay=False), required=True,
     help="Output directory for ssgsea results")
-@click.option('-n', '--pkl_file', type=click.Path(exists=True, dir_okay=False), required=True, 
+@click.option('-n', '--pkl_file', type=click.Path(exists=True, dir_okay=False),
     help="Path to network_integration.pkl.gz file from `mosaicmpi integrate` step")
-@click.option('-g', '--gmt_file', type=click.Path(exists=True, dir_okay=False), required=True, 
-    help="Path to GMT file with gene sets.")
+@click.option('-i', '--h5ad_file', type=click.Path(exists=True, dir_okay=False),
+    help="Path to .h5ad file from `mosaicmpi postprocess`")
+@click.option('-g', '--gene_sets', type=str, required=True, 
+    help="Path to GMT file with gene sets or Enrichr Library name.")
 @click.option("--min_intersection", type=int, default=5, 
     help="Minimum intersection size for gene sets")
 @click.option("--max_intersection", type=int, default=500, 
     help="Minimum intersection size for gene sets")
 @click.option('--cpus', type=int, default=cpus_available, show_default=True,
     help="Number of CPUs for MP-enabled tasks")
-def cmd_ssgsea(output_dir, pkl_file, gmt_file, min_intersection, max_intersection, cpus):
+def cmd_ssgsea(output_dir, pkl_file, h5ad_file, gene_sets, min_intersection, max_intersection, cpus):
     """
-    Compute and plot ssGSEA Normalized Enrichment Scores (NES) for programs from an integration.
+    Compute and plot ssGSEA Normalized Enrichment Scores (NES) for mosaicMPI programs. If a network_integration.pkl file
+    is provided, ssGSEA is performed on reprepresentative programs from an integration. If a .h5ad file is provided, ssGSEA
+    is performed on all programs.
     """
-    utils.start_logging()  # allows warning messages to be rinted even though logfile hasn't been made yet
+    utils.start_logging()  # allows warning messages to be printed even though logfile hasn't been made yet
     try:
         import gseapy
     except ImportError:
@@ -827,10 +928,7 @@ def cmd_ssgsea(output_dir, pkl_file, gmt_file, min_intersection, max_intersectio
                       "# Windows and MacOS_ARM64(M1/2-Chip)\n\t"
                       "pip install gseapy\nl"
                       )
-
-    # set CPU count for MP-enabled tasks
-    global cpus_available
-    cpus_available = cpus
+        sys.exit(1)
 
     # create directory structure, warn if not empty
     output_dir = os.path.normpath(output_dir)
@@ -841,32 +939,135 @@ def cmd_ssgsea(output_dir, pkl_file, gmt_file, min_intersection, max_intersectio
     # write to log file
     utils.start_logging(os.path.join(output_dir, "logfile.txt"))
 
-    network = Network.from_pkl(pkl_file)
-    raise NotImplementedError
+    if h5ad_file and not pkl_file:
+        # run ssGSEA on all programs from a .h5ad file
+        dataset = Dataset.from_h5ad("rna.h5ad")
+        programs = dataset.get_programs()
+        result = gseapy.ssgsea(data=programs, gene_sets=gene_sets, min_size=min_intersection, max_size=max_intersection, threads=cpus)
+        prog_nes = result.res2d.pivot(index="Term", columns="Name", values="NES")
+        prog_nes.columns = pd.MultiIndex.from_tuples(prog_nes.columns, names=("k", "program"))
+        prog_nes.to_csv(os.path.join(output_dir, "program_nes.txt"), sep="\t")
+    elif pkl_file and not h5ad_file:
+        # run ssGSEA on representative programs for each dataset in a network_integration.pkl file
+        network = Network.from_pkl(pkl_file)
+        rep_programs = network.get_representative_programs()
+        result = gseapy.ssgsea(data=rep_programs, gene_sets=gene_sets, min_size=min_intersection, max_size=max_intersection, threads=cpus)
+        rep_nes = result.res2d.pivot(index="Term", columns="Name", values="NES")
+        rep_nes.columns = pd.MultiIndex.from_tuples(rep_nes.columns, names=("community", "dataset", "k", "program"))
+        sorter = rep_nes.astype(float).idxmax(axis=1)
+        sorter = sorter.str[0].sort_values()
+        rep_nes = rep_nes.loc[sorter.index]
+        rep_nes.to_csv(os.path.join(output_dir, "representative_program_nes.txt"), sep="\t")
+
+        # plot ssGSEA NES scores by community and dataset
+        fig, figlegend = plot_representative_program_nes(network=network, rep_nes=rep_nes)
+        utils.save_fig(fig, os.path.join(output_dir, "representative_program_nes"))
+        utils.save_fig(figlegend, os.path.join(output_dir, "representative_program_nes.legend"))
+        
+    else:
+        logging.error("mosaicmpi ssgsea requires either a factorized dataset (.h5ad) file or a network_integration.pkl file to run ssGSEA on programs.")
+        sys.exit(1)
+    logging.info("All tasks completed successfully.")
     
 @click.command(name="compare-integrations")
 @click.option('-o', '--output_dir', type=click.Path(file_okay=False), required=True,
     help="Output directory for results")
-@click.option('-n', '--pkl_files', type=click.Path(exists=True, dir_okay=False), required=True, multiple=True,
-    help="Path to network_integration.pkl.gz files from `mosaicmpi integrate` step")
-@click.option('-d', '--dataset_mapping_toml', type=click.Path(exists=True, dir_okay=False), required=False,
-    help="Path to TOML file with dataset mappings in cases where dataset names are mismatched between integrations.")
+@click.option('--name1', type=str, default="Network 1",
+    help="Name of first network (plotted on the y-axis)")
+@click.option('--pkl1', type=click.Path(exists=True, dir_okay=False), required=True,
+    help="Path to first network_integration.pkl.gz file")
+@click.option('--name2', type=str, default="Network 2",
+    help="Name of second network (plotted on the x-axis)")
+@click.option('--pkl2', type=click.Path(exists=True, dir_okay=False), required=True,
+    help="Path to second network_integration.pkl.gz file")
+@click.option('-l', '--colors_toml', type=click.Path(dir_okay=False, exists=True), required=False,
+    help="TOML file with dataset_colors specification. If not provided, visually distinct colors will be chosen automatically.")
+def cmd_compare_integrations(output_dir, name1, pkl1, name2, pkl2, colors_toml):
+    """Compare communities from two network_integration.pkl files generated using `mosaicmpi integrate`.
 
-def cmd_compare_integrations(output_dir, pkl_files, dataset_mapping_toml):
-    """Compare communities from two network_integration.pkl files.
     """
-    raise NotImplementedError
+    utils.start_logging()  # allows warning messages to be printed even though logfile hasn't been made yet
+    
+    # create directory structure, warn if not empty
+    output_dir = os.path.normpath(output_dir)
+    os.makedirs(output_dir, exist_ok=True)
+    if os.listdir(output_dir):
+        logging.warning(f"{output_dir} is not empty. Files may be overwritten.")
 
-@click.command(name="label-transfer")
-@click.option('-o', '--output_dir', type=click.Path(file_okay=False), required=True,
-    help="Output directory for results")
+    # write to log file
+    utils.start_logging(os.path.join(output_dir, "logfile.txt"))
+    
+    network1 = Network.from_pkl(pkl1)
+    network2 = Network.from_pkl(pkl2)
+
+    jaccard = compare_community_jaccard_similarity(name1=name1, name2=name2, network1=network1, network2=network2)
+    jaccard.to_csv(os.path.join(output_dir, "jaccard.txt"), sep="\t")
+
+    # create or import a color for each dataset
+    if colors_toml is None:
+        colors = Colors()        
+    else:
+        logging.info("Using provided TOML file for color palettes")
+        colors = Colors.from_toml(colors_toml)
+
+    dataset_names = set(network1.integration.datasets) | set(network2.integration.datasets)
+    colors.add_missing_dataset_colors(datasets=dataset_names)
+    colors.to_toml(os.path.join(output_dir, "colors.toml"))
+
+    fig = plot_compare_integrations(name1, network1, name2, network2, colors=colors)
+    utils.save_fig(fig, os.path.join(output_dir, "jaccard_heatmap"))
+    logging.info("All tasks completed successfully.")
+
+@click.command(name="transfer-labels")
+@click.option("-o", '--output_dir', type=click.Path(file_okay=False, exists=False), default=os.getcwd(), show_default=True,
+    help="Output directory for results.")
 @click.option('-n', '--pkl_file', type=click.Path(exists=True, dir_okay=False), required=True,
     help="Path to network_integration.pkl.gz file from `mosaicmpi integrate` step")
+@click.option('-s', '--source', type=str, multiple=True,
+    help="Only calculate Name of source dataset for sample labels.")
+@click.option('-d', '--dest', type=str, multiple=True,
+    help="Name of destination dataset")
+@click.option('-l', '--layer', type=str, multiple=True,
+    help="Name of categorical data layer to transfer")
+@click.option('-a', '--annotate', type=str, required=False, multiple=True,
+    help="Annotate transfer heatmap using categorical data layer(s) from destination dataset")
+@click.option('-m', '--metadata_colors_toml', type=click.Path(dir_okay=False, exists=True), required=False,
+    help="TOML file with metadata_colors specification. If not provided, visually distinct colors will be chosen automatically.")
+def cmd_transfer_labels(output_dir, pkl_file, source, dest, layer, annotate, metadata_colors_toml):
+    """Transfer labels from a source to a destination dataset.
+    """ 
+    network = Network.from_pkl(pkl_file)
+    os.makedirs(output_dir, exist_ok=True)
 
-def cmd_label_transfer(output_dir, pkl_file):
-    """Transfer labels between datasets.
-    """
-    raise NotImplementedError
+    transfer_df = network.transfer_labels(source=(source if source else None),
+                                          dest=(dest if dest else None),
+                                          layer=(layer if layer else None))
+    transfer_df.to_csv(os.path.join(output_dir, "transfer_score.txt"), sep="\t")
+
+    if annotate:
+        if metadata_colors_toml:
+            colors = Colors.from_toml(metadata_colors_toml)
+        else:
+            colors = Colors.from_network(network)
+    else:
+        colors = None
+        annotate = None
+
+    if not source:
+        source = list(network.integration.datasets.keys())
+    if not dest:
+        dest = list(network.integration.datasets.keys())
+    if not layer:
+        layer = network.integration.get_metadata_df(subset_datasets=source, include_numerical=False).columns
+    
+    total = len(source) * len(dest) * len(layer)
+
+    for s, d, l in tqdm(product(source, dest, layer), total=total, unit="plot", desc="Creating plots"):
+        if len(network.integration.datasets[s].get_metadata_df()[l].unique()) > 1:
+            cgrid = plot_metadata_transfer(network=network, source=s, dest=d, layer=l, annotate=annotate, colors=colors)
+            cgrid.fig.suptitle(f"source: {s}, dest: {d}, layer: {l}")
+            cgrid.savefig(os.path.join(output_dir, f"s.{s}_d.{d}_l.{l}.pdf"))
+    logging.info("All tasks completed successfully.")
 
 cli.add_command(cmd_txt_to_h5ad)
 cli.add_command(cmd_update_h5ad_metadata)
@@ -882,4 +1083,4 @@ cli.add_command(cmd_create_config)
 cli.add_command(cmd_integrate)
 cli.add_command(cmd_ssgsea)
 cli.add_command(cmd_compare_integrations)
-cli.add_command(cmd_label_transfer)
+cli.add_command(cmd_transfer_labels)
