@@ -6,7 +6,7 @@ import pandas as pd
 import logging
 from datetime import datetime
 import sys
-from typing import Union, Optional
+from typing import Union, Optional, Literal
 from collections.abc import Iterable, Collection
 import os
 from glob import glob
@@ -16,7 +16,9 @@ import scanpy as sc
 import anndata as ad
 import pandas as pd
 import numpy as np
+from sklearn.impute import KNNImputer, SimpleImputer
 from statsmodels.gam.api import GLMGam, BSplines
+from tqdm import tqdm
 
 class Dataset():
     
@@ -87,6 +89,9 @@ class Dataset():
             self.is_normalized = is_normalized
             self.mosaicmpi_version = __version__
     
+        # fix limitations of AnnData on-disk format
+
+
     @classmethod
     def from_df(cls,
                   data: pd.DataFrame,
@@ -177,6 +182,15 @@ class Dataset():
     def is_normalized(self, value: bool):
         self.adata.uns["is_normalized"] = value
     
+    @property
+    def is_imputed(self):
+        """Outputs the imputation status of the dataset.
+
+        :return: True if dataset contains normalized data, False if it is raw data.
+        :rtype: bool
+        """
+        return "imputation" in self.adata.uns
+
     @property
     def patient_id_col(self):
         """Outputs the normalization status of the dataset.
@@ -192,8 +206,8 @@ class Dataset():
     @patient_id_col.setter
     def patient_id_col(self, value: str):
         
-        if value is not None and value not in self.adata.obs.columns:
-            avail_columns = ", ".join(self.adata.obs.columns)
+        if value is not None and value not in self.get_metadata_df():
+            avail_columns = ", ".join(self.get_metadata_df())
             raise ValueError(f"{value} is not a valid column in the metadata matrix. Available columns are: {avail_columns}")
         self.adata.uns["patient_id_col"] = value
 
@@ -257,18 +271,20 @@ class Dataset():
         if missing_samples_in_md:
             logging.warning("The following samples in the data (`adata.X`) were absent in the metadata:\n  - " + "\n  - ".join(missing_samples_in_md))
         self.adata.obs = obs.reindex(self.adata.obs.index)
+        self.append_to_history(f"metadata (adata.obs) updated.")
     
-    def get_metadata_type_summary(self):
-        """Return a printable summary of metadata and value types for each metadata layer.
+    def get_printable_metadata_type_summary(self):
+        """Return a printable summary of metadata and the types.
 
         :return: Summary of metadata
         :rtype: str
         """
-        msg = ""
+        msg = "Summary of metadata types\n\n    Sample metadata:\n"
         for col in self.adata.obs.columns:
-            msg += "    Column: " + col + "\n"
-            for value_type, count in self.adata.obs[col].dropna().map(type).value_counts().items():
-                msg += f"        {value_type}: {count}\n"
+            msg += "        " + col + ": " + self.adata.obs[col].dtype.name + "\n"
+        msg += "    Feature metadata:\n"
+        for col in self.adata.var.columns:
+            msg += "        " + col + ": " + self.adata.var[col].dtype.name + "\n"
         return msg
     
     def write_h5ad(self,
@@ -280,8 +296,9 @@ class Dataset():
         """
         filename = os.path.abspath(filename)
         logging.info(f"Writing to {filename}")
+        self.append_to_history("Writing to {filename}")
         self.adata.write_h5ad(filename)
-        logging.info(f"Done")
+        logging.info(f"Write completed.")
     
     def to_df(self,
               normalized: bool = False):
@@ -297,7 +314,7 @@ class Dataset():
             df = df.div(df.sum(axis=1), axis=0) * 1e6  # TPM normalization
         return df
         
-    def remove_unfactorizable_genes(self):
+    def remove_unfactorizable_features(self):
         """Removes genes with missing values or zero variance from the data matrix.
         """
         df = self.to_df(normalized=False)
@@ -308,19 +325,130 @@ class Dataset():
         logging.info(f"{n_missing} of {self.adata.n_vars} features are missing values.")
         if n_missing:
             logging.warning(f"Subsetting features to those with no missing values.")
+            self.append_to_history(f"Removing features with missing values: {genes_with_missingvalues}")
         # Check for genes with zero variance
         zerovargenes = (df.var() == 0)
         n_zerovar = zerovargenes.sum()
         logging.info(f"{n_zerovar} of {self.adata.n_vars} features have a variance of zero.")
         if n_zerovar:
             logging.warning(f"Subsetting features to those with nonzero variance.")
+            self.append_to_history(f"Removing features with zero variance: {zerovargenes}")
         
-
         genes_to_keep = (~genes_with_missingvalues) & (~zerovargenes)
-        
         self.adata = self.adata[:,genes_to_keep].copy()
 
-    def compute_gene_stats(self, odg_default_spline_degree: int = 3, odg_default_dof: int = 8, max_missingness: float = 0.5):
+    def cross_validate_imputation(self, imputer: Union[KNNImputer, SimpleImputer], n_folds: int = 100):
+        """Perform k-fold cross validation of imputation on the dataset without modifying the data.
+
+        :param imputer: imputer object from scikit-learn
+        :type imputer: Union[KNNImputer, SimpleImputer]
+        :param n_folds: number of folds, defaults to 100
+        :type n_folds: int, optional
+        :return: Datafram with statistics for each gene, including preimputation log-mean and log-variance, and NRMSD mean and variance across all folds.
+        :rtype: :class:`pandas.DataFrame`
+        """
+        logging.info(f"Performing k-fold cross-validation, with {n_folds} folds")
+        # k-fold cross-validation
+        rmsd_folds = []   # store gene-wise rmsd
+        data = self.adata.to_df()
+        random_numbers = np.random.random(data.shape)  # used to assign each data point to a fold
+        bins = np.linspace(0, 1, n_folds + 1)
+        for fold in tqdm(range(n_folds), unit="fold"):
+            lower, upper = bins[fold:fold + 2]
+            mask = (random_numbers >= lower) & (random_numbers < upper)
+            test = data.mask(~mask)
+            training = data.mask(mask)
+            imputed = imputer.fit_transform(training)
+            imputed = pd.DataFrame(imputed, index=data.index, columns=data.columns)
+            resid = (imputed - test)
+            rmsd = np.sqrt((resid ** 2).sum() / resid.notnull().sum())
+            rmsd_folds.append(rmsd)
+        rmsd_folds = pd.concat(rmsd_folds, axis=1)
+        nrmsd_folds = rmsd_folds.div(data.mean(), axis=0)
+        
+        # record stats in adata.var
+        cvstats = pd.DataFrame({
+            "log_mean_preimputation": np.log10(data.mean()),
+            "log_variance_preimputation": np.log10(data.var()),
+            "imputation_nrmsd_mean": nrmsd_folds.mean(axis=1),
+            "imputation_nrmsd_var": nrmsd_folds.mean(axis=1)
+        })
+
+        return cvstats
+
+    def impute_knn(self,
+                   n_neighbors: int = 5,
+                   weights: Literal["distance", "uniform"] = "distance",
+                   cross_validate: bool = True,
+                   n_folds: int = 100):
+        
+        """Imputation for completing missing values using k-Nearest Neighbors.
+           Each sample's missing values are imputed using the mean value from
+           `n_neighbors` nearest neighbors. Two samples are
+           close if the features that neither is missing are close. 
+
+        :param n_neighbors: Number of neighboring samples to use for imputation, defaults to 5
+        :type n_neighbors: int, optional
+        :param weights: Weight function used in prediction, defaults to 'distance'. Possible values:
+            - 'uniform' : uniform weights. All points in each neighborhood are
+            weighted equally.
+            - 'distance' : weight points by the inverse of their distance.
+            in this case, closer neighbors of a query point will have a
+            greater influence than neighbors which are further away.
+        :type weights: Literal[&quot;distance&quot;, &quot;uniform&quot;], optional
+        :param cross_validate: perform k-fold cross-validation, defaults to True
+        :type cross_validate: bool, optional
+        :param n_folds: number of folds for k-fold cross-validation, defaults to 100
+        :type n_folds: int, optional
+        """
+
+        logging.info(f"Imputing data with k-Nearest Neighbors, k={n_neighbors}, {weights} weights")
+
+        imputer = KNNImputer(n_neighbors=n_neighbors, weights=weights, keep_empty_features=True)
+        if n_folds < 2:
+            logging.warning(f"n_folds = {n_folds} is less than 2. Skipping cross-validation.")
+            cross_validate = False
+        if cross_validate:
+            cvstats = self.cross_validate_imputation(imputer=imputer, n_folds=n_folds)
+            self.adata.var = self.adata.var.join(cvstats, validate="1:1")
+            
+        self.adata.X = imputer.fit_transform(self.adata.X)
+        self.append_to_history(f"KNN Imputation: n_neighbors = {n_neighbors}, weights = {weights}")
+        self.adata.uns["imputation"] = {"method": "knn",
+                                        "n_neighbors": n_neighbors,
+                                        "weights": weights,
+                                        "cross_validation": cross_validate}
+        if cross_validate:
+            self.adata.uns["imputation"]["n_folds"] = n_folds
+
+    def impute_zeros(self,
+                     cross_validate: bool = True,
+                     n_folds: int = 100):
+
+        """Imputation by filling missing values with zeros.
+
+        :param cross_validate: perform k-fold cross-validation, defaults to True
+        :type cross_validate: bool, optional
+        :param n_folds: number of folds for k-fold cross-validation, defaults to 100
+        :type n_folds: int, optional
+        """
+        logging.info(f"Imputing data with zeros")
+        
+        imputer = SimpleImputer(strategy="constant", fill_value=0.0, keep_empty_features=True)
+        if n_folds < 2:
+            logging.warning(f"n_folds = {n_folds} is less than 2. Skipping cross-validation.")
+            cross_validate = False
+        if cross_validate:
+            cvstats = self.cross_validate_imputation(imputer=imputer, n_folds=n_folds)
+            self.adata.var = self.adata.var.join(cvstats, validate="1:1")
+        self.adata.X = imputer.fit_transform(self.adata.X)
+        self.append_to_history(f"Zero imputation")
+        self.adata.uns["imputation"] = {"method": "zero",
+                                        "cross_validation": cross_validate}
+        if cross_validate:
+            self.adata.uns["imputation"]["n_folds"] = n_folds
+
+    def model_overdispersed_genes(self, odg_default_spline_degree: int = 3, odg_default_dof: int = 8, max_missingness: float = 0.5):
         """
         Computes gene statistics and fits two models of mean and variance of genes in the dataset. The first method is the
         generalized additive model with smooth components (B-splines) to model the relationship of mean and variance
@@ -624,19 +752,19 @@ class Dataset():
         df.columns = df.columns.set_levels([l.astype("int") for l in df.columns.levels])
         if normalize:
             normalized = []
-            for _, subdf in df.groupby(axis=1, level=0):
-                normalized.append(subdf.div(subdf.sum(axis=1), axis=0))
-            df = pd.concat(normalized, axis=1)
+            for _, subdf in df.T.groupby(level=0):
+                normalized.append(subdf.div(subdf.sum()))
+            df = pd.concat(normalized).T
         if discretize:
             discretized = []
-            for _, subdf in df.groupby(axis=1, level=0):
-                discretized.append(subdf.eq(subdf.max(axis=1), axis=0).astype(int))
-            df = pd.concat(discretized, axis=1)        
+            for _, subdf in df.T.groupby(level=0):
+                discretized.append(subdf.eq(subdf.max()).astype(int))
+            df = pd.concat(discretized).T        
         if k is not None:
             df = df.loc[:, k]
-        df = df.sort_index(axis=0).sort_index(axis=1)   
+        df = df.sort_index(axis=1)   
         return df
-    
+
     def get_programs(self,
                  k: Union[int, Iterable] = None,
                  type="cnmf_gep_score"
@@ -684,6 +812,7 @@ class Dataset():
             raise ValueError(f"{unexplained_col_str} metadata columns have unrecognized dtypes.")
         df = self.adata.obs.select_dtypes(include=dtypes)
         df = df.dropna(axis=1, how="all")
+        df = df.replace("nan", np.NaN)
         return df
     
     def get_category_overrepresentation(self,
@@ -703,11 +832,11 @@ class Dataset():
         :rtype: pd.DataFrame
         """
         usage = self.get_usages(normalize=True).copy()
-        sample_to_class = self.get_metadata_df()[layer].replace("nan", np.NaN)
+        sample_to_class = self.get_metadata_df()[layer]
         if subset_categories is not None:
             sample_to_class[~sample_to_class.isin(subset_categories)] = np.NaN
         usage.index = usage.index.map(sample_to_class)
-        observed = usage.groupby(axis=0, level=0).sum()
+        observed = usage.groupby(level=0, observed=True).sum()
         observed = observed[observed.sum(axis=1) > 0]
         n_categories = observed.shape[0]
         if n_categories < 2:
@@ -717,10 +846,10 @@ class Dataset():
                                 f"Overrepresentation cannot be calculated with fewer than 2 categories for each layer. ")
             return pd.DataFrame(np.NaN, index = observed.index, columns=observed.columns)
         expected = []
-        for k, obs_k in observed.groupby(axis=1, level=1):
+        for k, obs_k in observed.T.groupby(level=1):
             exp_k = pd.DataFrame(obs_k.sum(axis=1)) @ pd.DataFrame(obs_k.sum(axis=0)).T / obs_k.sum().sum()
             expected.append(exp_k)
-        expected = pd.concat(expected, axis=1)
+        expected = pd.concat(expected).T
         chisq_resid = (observed - expected) / np.sqrt(expected)  # pearson residual of chi-squared test of contingency table
         if truncate_negative:
             chisq_resid = chisq_resid.clip(lower=0)
