@@ -24,7 +24,7 @@ class Integration():
         Integrate multiple datasets together.
 
         :param datasets: dictionary of name: Dataset pairs.
-        :type datasets: dict[str, Dataset]
+        :type datasets: dict[str, :class:`~mosaicmpi.dataset.Dataset`]
         :param corr_method: Correlation method: "pearson", "spearman", or "kendall", defaults to "pearson"
         :type corr_method: str, optional
         :param max_median_corr: Threshold for rank reduction procedure, relevant only for datasets where GEPs tend to be highly correlated.
@@ -103,7 +103,7 @@ class Integration():
         mapping = {}
         for dataset_name, dataset in self.datasets.items():
             if dataset.patient_id_col is not None:
-                for sample_id, patient_id in dataset.adata.obs[dataset.patient_id_col].replace("nan", np.NaN).items():
+                for sample_id, patient_id in dataset.get_metadata_df()[dataset.patient_id_col].items():
                     mapping[(dataset_name, sample_id)] = (dataset_name, patient_id)
         if mapping:
             return pd.Series(mapping)
@@ -319,7 +319,7 @@ class Integration():
                     df_filt = df
 
                 results = {}
-                for dataset_name, subdf in df_filt.groupby(axis=0, level=0):
+                for dataset_name, subdf in df_filt.groupby(level=0):
                     results[dataset_name] = subdf.shape[0]
                 nodetable[(node_filter, edge_filter)] = results
 
@@ -418,7 +418,7 @@ class Integration():
         
         combined = {}
         for dataset_name in subset_datasets:
-            if layer in self.datasets[dataset_name].adata.obs:
+            if layer in self.datasets[dataset_name].get_metadata_df():
                 combined[dataset_name] = self.datasets[dataset_name].get_category_overrepresentation(layer=layer,
                                                                                                      truncate_negative=truncate_negative,
                                                                                                      subset_categories=subset_categories)
@@ -452,7 +452,7 @@ class Integration():
         
         combined = {}
         for dataset_name in subset_datasets:
-            if layer in self.datasets[dataset_name].adata.obs:
+            if layer in self.datasets[dataset_name].get_metadata_df():
                 combined[dataset_name] = self.datasets[dataset_name].get_metadata_correlation(layer=layer, method=method)
         combined = pd.concat(combined)
         return combined
\ No newline at end of file
