@@ -199,12 +199,14 @@ class Network():
         """
         df = self.integration.get_category_overrepresentation(layer=layer,
                                                               subset_datasets=subset_datasets,
-                                                              truncate_negative=truncate_negative,
+                                                              truncate_negative=False,
                                                               subset_categories=subset_categories)
         mapper = {tuple([program.split("|")[0], int(program.split("|")[1]), int(program.split("|")[2])]): comm for program, comm in self.program_communities.items()}
         df.columns = df.columns.map(mapper)
-        df = df.groupby(axis=1, level=0).mean()
+        df = df.T.groupby(level=0).mean().T
         df = df.reindex(self.ordered_community_names, axis=1)
+        if truncate_negative:
+            df = df.clip(lower=0)
         return df
     
     def get_community_metadata_correlation(self,
@@ -227,7 +229,7 @@ class Network():
         ser = self.integration.get_metadata_correlation(layer=layer, subset_datasets=subset_datasets, method=method)
         mapper = {tuple([program.split("|")[0], int(program.split("|")[1]), int(program.split("|")[2])]): comm for program, comm in self.program_communities.items()}
         ser.index = ser.index.map(mapper)
-        ser = ser.groupby(axis=0, level=0).mean()
+        ser = ser.groupby(level=0).mean()
         ser = ser.reindex(self.ordered_community_names)
         return ser
 
@@ -486,7 +488,12 @@ class Network():
             layout = nx.spring_layout(self.program_graph, weight="community_weight", **kwargs)
             layout = {node: list(coords) for node, coords in layout.items()}
         elif algorithm == "umap":
-            import umap
+            try:
+                import umap
+            except ImportError:
+                raise ImportError("umap-learn is not installed. Please install using:\n\n\t"
+                            "conda install -c conda-forge umap-learn"
+                            )
             from sklearn.preprocessing import StandardScaler, RobustScaler
 
             programs = {}
@@ -620,9 +627,9 @@ class Network():
                                                   edge_attr=["n_edges", "weight"])
         self.comm_graph.add_nodes_from(self.communities.keys())
         
-    def get_representative_programs(self,
+    def get_representative_program_ids(self,
                          correlation_axis: Literal["programs", "usage"] = "programs"
-                         ):
+                         ) -> pd.Series:
         """Select programs based on correlation with the median of all programs in each community
         
         :param correlation_axis: axis on which to compute correlations between programs whether correlating the programs or the usages, defaults to "programs"
@@ -661,8 +668,8 @@ class Network():
 
         return selected_programs
 
-    def get_selected_rank_programs(self,
-                            k: Union[int, Dict[str, int]]):
+    def get_selected_rank_program_ids(self,
+                            k: Union[int, Dict[str, int]]) -> pd.Series:
         """Select programs based on rank. k may be either a single value for all datasets
         as an integer, or as a dict with separate values for each dataset.
 
@@ -684,6 +691,20 @@ class Network():
         selected_programs = selected_programs.sort_index().sort_values(key=lambda x: x.map(self.ordered_community_names.index))
         return selected_programs
 
+    def get_representative_programs(self,
+                         correlation_axis: Literal["programs", "usage"] = "programs"
+                         ) -> pd.DataFrame:
+        """Select programs based on correlation with the median of all programs in each community
+        
+        :param correlation_axis: axis on which to compute correlations between programs whether correlating the programs or the usages, defaults to "programs"
+        :type correlation_axis: int or dict
+        :return: Communities, indexed by the most central program for each dataset
+        :rtype: pd.Series
+        """
+        rep_programs_ids = self.get_representative_program_ids(correlation_axis=correlation_axis)
+        rep_programs = self.integration.get_programs()[rep_programs_ids.index]
+        rep_programs.columns = pd.MultiIndex.from_tuples([[community] + list(program_id) for community, program_id in zip(rep_programs_ids, rep_programs_ids.index)], names=["Community", "dataset", "k", "Program"])
+        return rep_programs
 
     def get_lowest_rank_programs(self,
                                 min_k: Optional[Union[int, Dict[str, int]]] = None,
@@ -715,7 +736,7 @@ class Network():
 
         selected_programs = []
         for community, cprograms in programs.groupby(programs):
-            for dataset_name, cdprograms in cprograms.groupby(axis=0, level=0):
+            for dataset_name, cdprograms in cprograms.groupby(level=0):
                 min_rank = cdprograms.index.get_level_values(1).min()
                 min_rank_programs = cdprograms[cdprograms.index.get_level_values(1) == min_rank]
                 selected_programs.append(min_rank_programs)
@@ -808,9 +829,9 @@ class Network():
         programs = programs.loc[:, ~programs.columns.to_frame().isnull().any(axis=1)]  # Remove programs not in any community
         # aggregate programs to community
         if method == "median":
-            community_scores = programs.groupby(axis=1, level=[0,1]).median()
+            community_scores = programs.T.groupby(level=[0,1]).median().T
         elif method == "mean":
-            community_scores = programs.groupby(axis=1, level=[0,1]).mean()
+            community_scores = programs.T.groupby(level=[0,1]).mean().T
         else:
             raise NotImplementedError
         community_scores = community_scores.loc[:, self.ordered_community_names]  # sort community names
@@ -818,36 +839,133 @@ class Network():
 
 
     def transfer_labels(self,
-                          source: str,
-                          dest: str,
-                          layer: str,
-                          subset_categories: Collection[str]
+                          source: Optional[Union[str, Collection[str]]] = None,
+                          dest: Optional[Union[str, Collection[str]]] = None,
+                          layer: Optional[Union[str, Collection[str]]] = None,
+                          subset_categories: Collection[str] = None,
+                          simplify: bool = True
                           ) -> pd.DataFrame:
-        """_summary_
+        """Transfer sample categories between datasets using usage of representative programs as a proxy.
 
-        :param source: Source dataset for label transfer
-        :type source: str
-        :param dest: Target dataset for label transfer
-        :type dest: str
-        :param layer: name of categorical data layer
-        :type layer: str
-        :param subset_categories: Provide a subset of categories for calculating overrepresentation
-        :type subset_categories: Collection[str]
-        :return: _description_
+        :param source: Source dataset(s) for label transfer, defaults to None
+        :type source: Union[str, Collection[str]], optional
+        :param dest: Target dataset(s) for label transfer, defaults to None
+        :type dest: Union[str, Collection[str]], optional
+        :param layer: name of categorical data layer(s) from source dataset, defaults to None
+        :type layer: Union[str, Collection[str]], optional
+        :param subset_categories: a subset of categories for calculating overrepresentation, defaults to None
+        :type subset_categories: Collection[str], optional
+        :param simplify: Simplify multi-index results when only one source, dest, or layer are specified, defaults to True
+        :type simplify: bool, optional
+        :raises ValueError: if source or dest is not a correct type
+        :return: transfer score
         :rtype: pd.DataFrame
         """
-        rprogs = self.get_representative_programs()  # representative programs
-        source_progs = rprogs.xs(source)
-        source_or = self.integration.datasets[source].get_category_overrepresentation(layer=layer, subset_categories=subset_categories)[source_progs.index]
-        source_or.columns = source_or.columns.map(source_progs)  # community-level overrepresentation based on representative programs
-        dest_progs = rprogs.xs(dest)
-        dest_usage = self.integration.datasets[dest].get_usages(normalize=False)
-        dest_cu = dest_usage[dest_progs.index]
-        dest_cu.columns = dest_cu.columns.map(dest_progs) # community-level usage based on representative programs
-        intgc = dest_cu.columns.intersection(source_or.columns)  # integrative communities (i.e., shared communities)
-        
-        source_or = source_or[intgc].div(source_or[intgc].sum(axis=1), axis=0)
-        dest_cu = dest_cu[intgc].div(dest_cu[intgc].sum(axis=1), axis=0)
 
-        transfer_df = source_or @ dest_cu[intgc].T  # multiply usage by overrepresentation for integrative communities
-        return transfer_df
+        # parameter flexibility, by default everything
+        if source is None:
+            sources = self.integration.datasets.keys()
+        elif isinstance(source, str):
+            sources = [source]
+        elif isinstance(source, Collection):
+            sources = source
+        else:
+            raise ValueError
+        
+        if layer is None:
+            # get all layers for all selected source datasets
+            layers = self.integration.get_metadata_df(include_numerical=False, subset_datasets=sources).columns
+        elif isinstance(layer, str):
+            layers = [layer]
+        elif isinstance(layer, Collection):
+            layers = layer
+        else:
+            raise ValueError
+        
+        if dest is None:
+            dests = self.integration.datasets.keys()
+        elif isinstance(dest, str):
+            dests = [dest]
+        elif isinstance(dest, Collection):
+            dests = dest
+        else:
+            raise ValueError
+        
+
+        agg = []
+        for source_name in sources:
+            for layer_name in layers:
+                rowblock = []
+                for dest_name in dests:
+                    rprogs = self.get_representative_program_ids()  # representative programs
+                    source_progs = rprogs.xs(source_name)
+                    source_or = self.integration.datasets[source_name].get_category_overrepresentation(layer=layer_name, subset_categories=subset_categories)[source_progs.index]
+                    source_or.columns = source_or.columns.map(source_progs)  # community-level overrepresentation based on representative programs
+                    dest_progs = rprogs.xs(dest_name)
+                    dest_usage = self.integration.datasets[dest_name].get_usages(normalize=False)
+                    dest_cu = dest_usage[dest_progs.index]
+                    dest_cu.columns = dest_cu.columns.map(dest_progs) # community-level usage based on representative programs
+                    intgc = dest_cu.columns.intersection(source_or.columns)  # integrative communities (i.e., shared communities)
+                    
+                    source_or = source_or[intgc].div(source_or[intgc].sum(axis=1), axis=0)
+                    dest_cu = dest_cu[intgc].div(dest_cu[intgc].sum(axis=1), axis=0)
+
+                    transfer_df = source_or @ dest_cu[intgc].T  # multiply usage by overrepresentation for integrative communities
+                    transfer_df.index = pd.MultiIndex.from_arrays([[source_name]*transfer_df.shape[0],
+                                                                   [layer_name]*transfer_df.shape[0],
+                                                                   transfer_df.index],
+                                                                   names=["source_dataset", "layer", "category"])
+
+                    transfer_df.columns = pd.MultiIndex.from_arrays([[dest_name]*transfer_df.shape[1], transfer_df.columns], names=["dest_dataset", "sample"])
+                    rowblock.append(transfer_df)
+                agg.append(pd.concat(rowblock, axis=1))
+        agg = pd.concat(agg)
+
+        if simplify:
+            if isinstance(layer, str):
+                agg = agg.droplevel(axis=0, level="layer")
+            if isinstance(source, str):
+                agg = agg.droplevel(axis=0, level="source_dataset")
+            if isinstance(dest, str):
+                agg = agg.droplevel(axis=1, level="dest_dataset")
+        return agg
+
+
+def compare_community_jaccard_similarity(name1: str, network1: Network, name2: str, network2: Network, subset_to_shared_datasets: bool = True):
+    """Calculates the jaccard similarity of communities between two networks with overlapping nodes. One use case is to compare integrations
+    after adding new datasets; the other is to compare integrations at different community resolutions.
+
+    :param name1: name of first network
+    :type name1: str
+    :param network1: first network
+    :type network1: :class:`mosaicmpi.Network`
+    :param name2: name of second network
+    :type name2: str
+    :param network2: second network
+    :type network2: :class:`mosaicmpi.Network`
+    :param subset_to_shared_datasets: calculate jaccard similarity over shared datasets only, defaults to True
+    :type subset_to_shared_datasets: bool, optional
+    """
+
+    all_datasets = set(network1.integration.datasets) | set(network2.integration.datasets)
+    shared_datasets = set(network1.integration.datasets) & set(network2.integration.datasets)
+    net1_datasets = set(network1.integration.datasets) - shared_datasets
+    net2_datasets = set(network2.integration.datasets) - shared_datasets
+
+    jaccard = pd.DataFrame(np.NaN,
+                           index=pd.Index(network1.ordered_community_names, name=name1),
+                           columns=pd.Index(network2.ordered_community_names, name=name2))
+    for net1_comm, net1_nodes in network1.communities.items():
+        for net2_comm, net2_nodes in network2.communities.items():
+            
+            intersection = len(net2_nodes & net1_nodes)
+            if subset_to_shared_datasets:
+                # jaccard is calculated based on shared datasets only
+                union = len(set(n for n in (net1_nodes | net2_nodes) if node_to_program(n)[0] in shared_datasets))
+            else:
+                union = len(net1_nodes | net2_nodes)
+            jaccard.loc[net1_comm, net2_comm] = intersection / union
+
+    jaccard = jaccard[jaccard.idxmax(0).astype(int).sort_values().index]
+
+    return jaccard
\ No newline at end of file
