@@ -2,12 +2,12 @@
 from .dataset import Dataset
 from .integration import Integration
 from .colors import Colors
-from .network import Network
+from .network import Network, compare_community_jaccard_similarity
 from . import utils
 
 
 from collections.abc import Iterable, Collection, Mapping
-from typing import Union, Optional, Literal
+from typing import Union, Optional, Literal, Dict
 
 import pandas as pd
 import numpy as np
@@ -30,7 +30,7 @@ import networkx as nx
 def plot_feature_missingness(dataset: Dataset, ax: Optional[Axes] = None, proportion=False):
         
     if ax is None:
-        fig, ax_plot = plt.subplots(figsize=[10, 3], layout="constrained")
+        fig, ax_plot = plt.subplots(figsize=[10, 4], layout="constrained")
     else:
         ax_plot = ax
 
@@ -39,8 +39,12 @@ def plot_feature_missingness(dataset: Dataset, ax: Optional[Axes] = None, propor
     else:
         stat = "missing_values"
 
-    missingness = dataset.adata.var[stat]
-    missingness.value_counts().sort_index().plot.bar(width=0.9, color="k", ax=ax_plot)
+    if proportion:
+        missingness = dataset.adata.var["missingness"]
+        missingness.plot.hist(bins=100, color="k", ax=ax_plot)
+    else:
+        missing_values = dataset.adata.var["missing_values"]
+        missing_values.value_counts().sort_index().plot.bar(width=0.9, color="k", ax=ax_plot)
     ax_plot.set_ylabel("Features")
     ax_plot.set_xlabel(stat)
 
@@ -48,19 +52,32 @@ def plot_feature_missingness(dataset: Dataset, ax: Optional[Axes] = None, propor
         return fig
 
 
-
 def plot_feature_dispersion(dataset: Dataset,
                             show_selected: bool = False,
                             show_model_curve: bool = True,
                             y_unit: Literal["log_variance", "odscore", "log_odscore", "vscore", "log_vscore"] = "log_variance",
-                            exclude_unmodelled_features: bool = True,
+                            modelled_features: bool = True,
+                            unmodelled_features: bool = False,
                             ax: Optional[Axes] = None):
+
     
-    df = dataset.adata.var.sort_values("mean")
+    df = dataset.adata.var
+    df = df.sort_values("mean")
+    model_curve_data = df.copy()
+
+    if not modelled_features and not unmodelled_features:
+        raise ValueError("at least one of modelled_features and unmodelled_features must be True.")
+    elif modelled_features and not unmodelled_features:
+        df = df[~df["odscore_excluded"]]
+    elif not modelled_features and unmodelled_features:
+        df = df[df["odscore_excluded"]]
+    elif modelled_features and unmodelled_features:
+        pass
+
     if "log_odscore" not in df.columns:  # required for compatibility with older h5ad files which do not have this column
         df["log_odscore"] = np.log10(df["odscore"])
         df["log_vscore"] = np.log10(df["vscore"])
-    
+
     if ax is None:
         fig, ax_plot = plt.subplots(figsize=[4, 4], layout="tight")
     else:
@@ -80,7 +97,7 @@ def plot_feature_dispersion(dataset: Dataset,
     # Show the model curve from the GAM
     if show_model_curve:
         if y_unit == "log_variance":
-            ax_plot.plot(df["log_mean"], df["gam_fittedvalues"], color="green")
+            ax_plot.plot(model_curve_data["log_mean"], model_curve_data["gam_fittedvalues"], color="green")
         elif y_unit == "odscore":
             ax_plot.axhline(1)
         elif y_unit == "log_odscore":
@@ -105,8 +122,6 @@ def plot_feature_dispersion(dataset: Dataset,
         return fig
 
 
-   
-
 def plot_feature_overdispersion_histogram(dataset: Dataset,
                                           show_selected: bool = False,
                                           y_unit: Literal["odscore", "log_odscore"] = "log_variance",
@@ -230,7 +245,7 @@ def annotated_heatmap(
                 ordered_rgb[~ordered_rgb.isin(metadata_colors[track])] = np.NaN
                 ordered_rgb = ordered_rgb.replace(metadata_colors[track])
                 if ordered_rgb.isnull().any():
-                    if pd.api.types.is_categorical_dtype(annot):
+                    if pd.api.types.is_categorical_dtype(annot) and missing_data_color not in ordered_rgb.cat.categories:
                         ordered_rgb = ordered_rgb.cat.add_categories(missing_data_color)
                     ordered_rgb = ordered_rgb.fillna(missing_data_color)
                 ordered_rgb = ordered_rgb.astype("object").map(mpl.colors.to_rgb)
@@ -259,9 +274,9 @@ def annotated_heatmap(
     plt.colorbar(im_heatmap, ax=ax, location="top", label=cbar_label)
     return fig
 
-def plot_usage_heatmap(dataset: Dataset, k: Optional[int], colors, subset_metadata = None, subset_samples = None, title = None, cluster_programs = False, cluster_samples = True, show_sample_labels = True):
+def plot_usage_heatmap(dataset: Dataset, k: Optional[int], colors: Colors, normalize_usage: bool = True, subset_metadata = None, subset_samples = None, title: str = None, cluster_programs = False, cluster_samples = True, show_sample_labels = True):
     assert dataset.has_cnmf_results
-    df = dataset.get_usages(k=k)
+    df = dataset.get_usages(k=k, normalize=normalize_usage)
     if subset_samples is not None:
         df = df.loc[subset_samples]
     samples = df.index.to_series()
@@ -269,7 +284,6 @@ def plot_usage_heatmap(dataset: Dataset, k: Optional[int], colors, subset_metada
     if subset_metadata is not None:
         metadata = metadata.loc[:, subset_metadata]
     metadata_colors = {col: colors.get_metadata_colors(col) for col in metadata.columns}
-    df = df.div(df.sum(axis=1), axis=0)
     fig = annotated_heatmap(data=df, metadata=metadata,
                             metadata_colors=metadata_colors, 
                             missing_data_color=colors.missing_data_color, 
@@ -285,7 +299,7 @@ def plot_sample_numbers(dataset: Dataset, layer: str, figsize = None, ax = None)
     """Bar plot of sample numbers in each category based on a single categorical layer.
 
     :param dataset: dataset
-    :type dataset: Dataset
+    :type dataset: :class:`~mosaicmpi.dataset.Dataset`
     :param layer: categorical metadata layer
     :type layer: str
     :param figsize: figure size, defaults to None
@@ -297,14 +311,11 @@ def plot_sample_numbers(dataset: Dataset, layer: str, figsize = None, ax = None)
     """
 
 
-    data = dataset.get_metadata_df(include_numerical=False)[layer].replace("nan", np.NaN).value_counts()
+    data = dataset.get_metadata_df(include_numerical=False)[layer].value_counts()
     
     if ax is None:
         if figsize is None:
             figsize = [1 + len(data) / 4, 4]
-
-
-
         fig, ax_plot = plt.subplots(figsize=figsize, layout="constrained")
     else:
         ax_plot = ax
@@ -359,7 +370,7 @@ def plot_pairwise_corr(integration: Integration, subplot_size: Collection = [3,
     """Plot histograms of the pairwise correlation distribution for each dataset pair.
 
     :param integration: Integration object
-    :type integration: Integration
+    :type integration: :class:`~mosaicmpi.integration.Integration`
     :param subplot_size: width and height of each subplot, defaults to [3, 3.5]
     :type subplot_size: Collection, optional
     :param overlaid: Overlay negative and positive parts to compare the distribution, defaults to False
@@ -487,7 +498,7 @@ def plot_overdispersed_features_upset(integration: Integration, figsize: Collect
     """Plot overlaps of overdispersed features between datasets
 
     :param integration: integration object
-    :type integration: Integration
+    :type integration: :class:`~mosaicmpi.integration.Integration`
     :param figsize: width and height of figure, defaults to [6, 4]
     :type figsize: Collection, optional
     :return: figure
@@ -503,7 +514,7 @@ def plot_features_upset(integration: Integration, figsize=[6, 4], show_counts: b
     """Plot overlaps of features between datasets
 
     :param integration: integration object
-    :type integration: Integration
+    :type integration: :class:`~mosaicmpi.integration.Integration`
     :param figsize: width and height of figure, defaults to [6, 4]
     :type figsize: Collection, optional
     :return: figure
@@ -589,7 +600,7 @@ def plot_community_usage_per_sample(network: Network,
 
     fig, axes = plt.subplots(n_row, n_col, figsize=[3 * n_col, 3 * n_row], sharex=True, sharey=True, layout="tight")
 
-    for i, (celltype, subdf) in enumerate(df.groupby(axis=0, level=0)):
+    for i, (celltype, subdf) in enumerate(df.groupby(level=0)):
         ax = axes[i // n_col, i % n_col]
         sns.violinplot(data=subdf, linewidth=0.1, ax=ax, cut=0, color=colors.community_colors)
         ax.set_title(celltype)
@@ -603,10 +614,10 @@ def plot_community_contribution(network: Network, colors: Colors, figsize: Colle
     """_summary_
 
     :param network: _description_
-    :type network: Network
+    :type network: :class:`~mosaicmpi.network.Network`
     :param colors: _description_
-    :type colors: Colors
-    :param figsize: _description_, defaults to None
+    :type colors: :class:`~mosaicmpi.colors.Colors`
+    :param figsize: figure size, defaults to None
     :type figsize: Collection, optional
     :param highlight_central_program: _description_, defaults to True
     :type highlight_central_program: bool, optional
@@ -616,15 +627,13 @@ def plot_community_contribution(network: Network, colors: Colors, figsize: Colle
     :rtype: _type_
     """
 
-
-
     marker_style = {
         1: ("s", 30),  # 1 factor: square markers, size 30
-        2: (2, 30)     # 2 factors: marker #2 (up tick), size 30
+        2: (2, 30),     # 2 factors: marker #2 (up tick), size 30
         }
 
     if highlight_central_program:
-        central_ranks = pd.DataFrame(network.get_representative_programs()).reset_index().set_index(["Community", "dataset"])["k"]
+        central_ranks = pd.DataFrame(network.get_representative_program_ids()).reset_index().set_index(["Community", "dataset"])["k"]
 
     n_datasets = network.integration.n_datasets
 
@@ -637,11 +646,18 @@ def plot_community_contribution(network: Network, colors: Colors, figsize: Colle
             raise ValueError(f"{orientation} is not a valid orientation. Please choose from `horizontal` or `vertical`")
 
     if orientation == "horizontal":
-        fig, axes = plt.subplots(1, n_datasets+1, figsize=figsize, sharex=True, sharey=True, layout="tight")
+        fig, axes = plt.subplots(1, n_datasets+1, figsize=figsize, sharex=False, sharey=True, layout="tight")
     elif orientation == "vertical":
-        fig, axes = plt.subplots(n_datasets+1, 1, figsize=figsize, sharex=True, sharey=True, layout="tight")
+        fig, axes = plt.subplots(n_datasets+1, 1, figsize=figsize, sharex=False, sharey=True, layout="tight")
     
+    all_k_values = set()
+    for selected_k_values in network.integration.selected_k.values():
+        all_k_values |= set(selected_k_values)
+    all_k_values = sorted(list(all_k_values))
+
     for dataset, ax in zip(network.integration.datasets, axes):
+        selected_k_values = network.integration.selected_k[dataset]
+        x_positions = [all_k_values.index(k) for k in selected_k_values]
         for y, community in enumerate(network.ordered_community_names):
             members = network.communities[community]
             counts = pd.Series([m.rpartition("|")[0] for m in members]).value_counts()
@@ -649,7 +665,7 @@ def plot_community_contribution(network: Network, colors: Colors, figsize: Colle
             # plot line if any factors are present
             line_x = []
             line_y = []
-            for x, rank in enumerate(network.integration.selected_k[dataset]):
+            for x, rank in zip(x_positions, selected_k_values):
                 line_x.append(x)
                 if f"{dataset}|{rank}" in counts.index:
                     line_y.append(y)
@@ -661,22 +677,19 @@ def plot_community_contribution(network: Network, colors: Colors, figsize: Colle
                 # plot different markers depending on how many factors are present:
                 scatter_x = []
                 scatter_y = []
-                for x, rank in enumerate(network.integration.selected_k[dataset]):
+                for x, rank in zip(x_positions, selected_k_values):
                     factor_prefix = f"{dataset}|{rank}"
                     if factor_prefix in counts.index and counts[factor_prefix] == count:
                         scatter_x.append(x)
                         scatter_y.append(y)
                 ax.scatter(scatter_x, scatter_y, color=colors.dataset_colors[dataset], marker=style[0], s=style[1])
-            
+
             if highlight_central_program and (community, dataset) in central_ranks:
-                x = network.integration.selected_k[dataset].index(central_ranks[(community, dataset)])
+                x = all_k_values.index(central_ranks[(community, dataset)])
                 ax.add_patch(Rectangle((x - 0.4, y-0.4), width = 0.8, height = 0.8, edgecolor="k", facecolor='none'))
-
-        ax.set_yticks(list(range(len(network.ordered_community_names))))
-        ax.set_yticklabels(network.ordered_community_names)
-        ax.set_xticks(list(range(len(network.integration.selected_k[dataset]))))
-        ax.set_xticklabels(network.integration.selected_k[dataset])
-
+        ax.set_yticks(list(range(len(network.ordered_community_names))), labels=network.ordered_community_names)
+        ax.set_xticks(x_positions, labels=selected_k_values)
+        ax.set_xlim(-0.6, len(all_k_values) - 0.4)
 
         if orientation == "vertical":
             ax.set_ylabel("Community")
@@ -689,8 +702,6 @@ def plot_community_contribution(network: Network, colors: Colors, figsize: Colle
                 ax.set_ylabel("Community")
 
         ax.set_title(dataset)
-   
-
 
     # Add legend
     cbdrlegend = []
@@ -708,7 +719,6 @@ def plot_community_contribution(network: Network, colors: Colors, figsize: Colle
 #   Helper functions for circle bar plots #
 ###########################################
 
-
 def draw_circle_bar_plot(position, enrichments, colors, size, ax, scale_factor: float=1, draw_labels: bool=False, label_font_size: float=1):
 
     x, y = position
@@ -817,7 +827,7 @@ def plot_overrepresentation_program_network(network: Network,
         draw_circle_bar_plot(
             position=(0, 0.5),
             enrichments=pd.Series(max_or, index=overrepresentation.index),
-            colors=color_list,
+            colors=overrepresentation.index.map(colors.get_metadata_colors(layer)),
             scale_factor=scale_factor,
             size=legend_pie_size,
             draw_labels=True,
@@ -1071,7 +1081,7 @@ def plot_program_network_npatients(network: Network,
         raise ValueError("No samples have valid patient IDs. Make sure to set the patient_id_col property for each Dataset")
     usages = usages[usages.index.isin(s2p.index)]
     usages.index = usages.index.map(s2p)
-    usages = usages.groupby(axis=0, level=[0,1]).any()
+    usages = usages.groupby(level=[0,1]).any()
         
     labels = usages[network.programs_in_graph].sum().apply(lambda x: str(int(x))).to_dict()
     labels = {f"{k[0]}|{k[1]}|{k[2]}": v for k,v in labels.items()}
@@ -1240,7 +1250,7 @@ def plot_overrepresentation_community_network(network: Network,
         # Add legends
         draw_circle_bar_plot(
             position=(0, 0.5),
-            enrichments=pd.Series(max_or, index=comm_or.index.sort_values().unique()),
+            enrichments=pd.Series(max_or, index=overrepresentation.index),
             colors=overrepresentation.index.map(colors.get_metadata_colors(layer)),
             scale_factor=scale_factor,
             size=legend_pie_size,
@@ -1434,8 +1444,8 @@ def plot_overrepresentation_community_bar(network: Network,
     # if existing axes object is provided, plots with legend on that axes. Otherwise, creates a new figure with a separate plot and legend Axes.
     if ax is None:
         if figsize is None:
-            figsize = [0.1 * df.shape[1] + 4, 4]
-        fig, (ax_plot, ax_legend) = plt.subplots(1, 2, figsize=figsize, sharey=True, gridspec_kw={"width_ratios": [4, 1]}, layout="tight")
+            figsize = [0.2 * df.shape[1] + 4, 4]
+        fig, (ax_plot, ax_legend) = plt.subplots(1, 2, figsize=figsize, sharey=True, gridspec_kw={"width_ratios": [df.shape[1]/5, 1]}, layout="tight")
         ax_legend.set_axis_off()
     else:
         ax_plot = ax
@@ -1443,7 +1453,7 @@ def plot_overrepresentation_community_bar(network: Network,
         
     # Overrepresentation plot
     df.T.plot.bar(stacked=True, ax=ax_plot, width = 0.9, color=colors.get_metadata_colors(layer), legend=False)
-    ax_plot.set_xticklabels(ax_plot.get_xticklabels(), rotation=0)
+    ax_plot.set_xticklabels(ax_plot.get_xticklabels(), rotation=90)
     ax_plot.set_ylabel("Median overrepresentation")
     ax_plot.set_xlabel("Community")
     
@@ -1474,7 +1484,7 @@ def plot_metadata_correlation_community_bar(network: Network,
         
     # Overrepresentation plot
     md_corr.plot.bar(ax=ax_plot, width = 0.9, color="#888888")
-    ax_plot.set_xticklabels(ax_plot.get_xticklabels(), rotation=0)
+    ax_plot.set_xticklabels(ax_plot.get_xticklabels(), rotation=90)
     ax_plot.set_ylabel(f"Median {method.capitalize()} Correlation")
     ax_plot.set_xlabel("Community")
     
@@ -1550,4 +1560,184 @@ def plot_sample_entropy(network: Network,
     ax.set_ybound(lower=0)
     ax.set_title("Shannon Entropy")
     ax.set_xticklabels(ax.get_xticklabels(), rotation=90)
+    return fig
+
+
+
+########################
+# Label-transfer plots #
+########################
+
+
+def plot_metadata_transfer(network: Network, source: str, dest: str, layer: str, figsize: Collection[float, float] = None, annotate: Optional[Union[str, Collection[str]]] = None, colors: Colors = None) -> Figure:
+    """Plot heatmap of metadata transfer scores
+
+    :param network: network through which to propagate labels
+    :type network: :class:`mosaicmpi.network.Network`
+    :param colors: colors object with metadata colors if 'annotation' parameter is specified
+    :type colors: :class:`mosaicmpi.colors.Colors`
+    :param source: Source dataset for label transfer
+    :type source: str
+    :param dest: Target dataset for label transfer
+    :type dest: str
+    :param layer: name of categorical data layer from source dataset
+    :type layer: str
+    :param annotate: categorical metadata layer, defaults to None
+    :type annotate: Union[str, Collection[str]], optional
+    :return: figure object
+    :rtype: Figure
+    """
+    if annotate is None:
+        annotations = []
+    elif isinstance(annotate, str):
+        annotations = [annotate]
+    elif isinstance(annotate, Collection):
+        annotations = annotate
+
+    transfer_df = network.transfer_labels(source=source, dest=dest, layer=layer)
+    if annotate is not None:
+        metadata = network.integration.datasets[dest].get_metadata_df()[annotations].copy()
+        for annotation_layer in annotations:
+            mapper = colors.get_metadata_colors(annotation_layer)
+            color_vec = metadata[annotation_layer].map(mapper)
+            if colors.missing_data_color not in color_vec.cat.categories:
+                color_vec = color_vec.cat.add_categories(colors.missing_data_color)
+            color_vec = color_vec.fillna(colors.missing_data_color)
+            metadata[annotation_layer] = color_vec
+        col_colors = metadata
+    else:
+        col_colors = None
+    if figsize is None:
+        figsize = [8, 1 + transfer_df.shape[0] * 0.2 + len(annotations) * 0.3]
+
+    fig = sns.clustermap(data = transfer_df, col_colors=col_colors, figsize=figsize, row_cluster=False, xticklabels=False, yticklabels=True, cmap="Blues")
+    return fig
+
+
+    
+################
+# ssGSEA plots #
+################
+
+
+def plot_representative_program_nes(network: Network, rep_nes: pd.DataFrame):
+
+    rep_ids = network.get_representative_program_ids()
+    height_ratios = rep_ids.value_counts()[network.ordered_community_names].values
+    ds_names = sorted(list(network.integration.datasets.keys()))
+
+    df = rep_nes.T.copy()
+    df = df.droplevel(axis=0, level=[2,3])
+    figsize = [df.shape[1] * 0.2, df.shape[0] * 0.2 + 4]
+
+    fig, axes = plt.subplots(nrows = len(height_ratios), figsize=figsize, sharex=True, gridspec_kw={"height_ratios": height_ratios}, layout="constrained")
+    figlegend, axlegend = plt.subplots(figsize=[1,2], layout="tight")
+    for community, ax in zip(network.ordered_community_names, axes):
+        df_comm = df.loc[community].astype("float").sort_index(key = lambda x: x.map(ds_names.index))
+        sns.heatmap(df_comm, square=True, yticklabels=True, cmap="RdBu_r", vmin = -0.5, vmax=0.5, center = 0, xticklabels=True, cbar_ax=axlegend, ax=ax)
+        ax.set_yticklabels(ax.get_yticklabels(), rotation=0, )
+        ax.set_xlabel(None)
+        ax.set_ylabel(community, rotation=0, fontweight = "bold", fontsize="large", va="center", ha="right")
+        is_bottom_subplot = community == network.ordered_community_names[-1]
+        ax.tick_params(top=False,
+                bottom=False,
+                left=False,
+                right=False,
+                labelleft=False,
+                labelright=True,
+                labelbottom=is_bottom_subplot)
+    fig.supylabel(f"Community")
+    return fig, figlegend
+
+####################
+# Compare Networks #
+####################
+
+def plot_compare_integrations(name1: str, network1: Network, name2: str, network2: str, colors: Colors, figsize: Optional[Collection[float]] = None) -> Figure:
+    """Calculate jaccard similarity of communities from two Network objects.
+       Jaccard similarity is calculated over programs from datasets in common between the two Network objects,
+       with network 1 on the y-axis, and network 2 on the x-axis.
+
+    :param name1: name of first network
+    :type name1: str
+    :param network1: first network
+    :type network1: :class:`mosaicmpi.Network`
+    :param name2: name of second network
+    :type name2: str
+    :param network2: second network
+    :type network2: :class:`mosaicmpi.Network`
+    :param colors: colors object with a color for each dataset from both networks.
+    :type colors: Colors
+    :param figsize: (width, height), defaults to None
+    :type figsize: Optional[Collection[float]], optional
+    :return: figure
+    :rtype: Figure
+    """
+    
+    shared_datasets = set(network1.integration.datasets) & set(network2.integration.datasets)
+    net1_datasets = set(network1.integration.datasets) - shared_datasets
+    net2_datasets = set(network2.integration.datasets) - shared_datasets
+
+    ds_colors = {"shared": colors.missing_data_color} | {ds: colors for ds, colors in colors.dataset_colors.items() if ds in (net1_datasets | net2_datasets)}
+
+    jaccard = compare_community_jaccard_similarity(name1=name1, network1=network1, name2=name2, network2=network2)
+
+    net1_bars = pd.DataFrame(np.NaN, index=jaccard.index[::-1], columns=["shared"] + list(net1_datasets))
+    for net1_comm in network1.ordered_community_names:
+        net1_bars.loc[net1_comm, "shared"] = len(set(n for n in network1.communities[net1_comm] if utils.node_to_program(n)[0] in shared_datasets))
+        for d in net1_datasets:
+            net1_bars.loc[net1_comm, d] = len(set(n for n in network1.communities[net1_comm] if utils.node_to_program(n)[0] == d))
+    net1_bars
+
+    net2_bars = pd.DataFrame(np.NaN, index=jaccard.columns, columns=["shared"] + list(net2_datasets))
+    for net2_comm in network2.ordered_community_names:
+        net2_bars.loc[net2_comm, "shared"] = len(set(n for n in network2.communities[net2_comm] if utils.node_to_program(n)[0] in shared_datasets))
+        for d in net2_datasets:
+            net2_bars.loc[net2_comm, d] = len(set(n for n in network2.communities[net2_comm] if utils.node_to_program(n)[0] == d))
+    net2_bars
+
+    max_bar_height = max(net1_bars.T.sum().max(), net2_bars.T.sum().max())
+
+    if figsize is None:
+        figsize = [0.5 * jaccard.shape[1] + 2, 0.5 * jaccard.shape[0] + 3]
+
+    fig, axes = plt.subplots(3,3, figsize=figsize, gridspec_kw={"width_ratios": [10, 1, 0.2], "height_ratios": [1, 1, 10]},
+                            sharex="col", sharey="row", layout="constrained")
+
+    sns.heatmap(jaccard, xticklabels=True, cmap="Blues", ax=axes[2,0], cbar_ax=axes[0,2])
+    axes[2,0].set_yticklabels(axes[2,0].get_yticklabels(), rotation=0)
+    axes[2,0].set_ylabel(f"{name1}\ncommunity")
+    axes[2,0].set_xlabel(f"{name2}\ncommunity")
+
+    axes[0,2].set_title("Jaccard\nsimilarity")
+
+
+    # horizontal bars
+    y_tick_pos = [i + 0.5 for i in range(len(network1.ordered_community_names))]
+    left = pd.Series(0, index=net1_bars.index[::-1])
+    for ds, ds_width in net1_bars[::-1].items():
+        axes[2,1].barh(y=y_tick_pos, width=ds_width, left=left, height=0.8, color=ds_colors[ds])
+        left += ds_width
+    axes[2,1].set_xlim([0,max_bar_height])
+    axes[2,1].set_xlabel("Nodes")
+
+    # vertical bars
+    x_tick_pos = [i + 0.5 for i in range(len(network2.ordered_community_names))]
+    bottom = pd.Series(0, index=net2_bars.index)
+    for ds, ds_height in net2_bars.items():
+        axes[1,0].bar(x=x_tick_pos, height=ds_height, bottom=bottom, width=0.8, color=ds_colors[ds])
+        bottom += ds_height
+    axes[1,0].set_ylim([0,max_bar_height])
+    axes[1,0].set_ylabel("Nodes")
+
+
+    axes[0,0].set_axis_off()
+    axes[0,1].set_axis_off()
+    axes[1,1].set_axis_off()
+    axes[1,2].set_axis_off()
+    axes[2,2].set_axis_off()
+
+    legend_handles = [Patch(color=color, label=ds) for ds, color in ds_colors.items()]
+    axes[0,0].legend(handles=legend_handles, loc = "center", ncols = 3)
+    
     return fig
\ No newline at end of file
